rm(list=ls())

donnees <- read.table("only_quali.txt",header=T,sep="\t")
str(donnees)


#pour l'apprentissage
myfit <- function(formule,data){
  #vrification
  if (!inherits(formule,"formula")) {formule <- as.formula(formule)}
  #data.frame
  D <- model.frame(formule,data)
  #y
  y <- D[,1]
  #X
  allX <- D[,-1]
  #calcul des tableaux croiss
  tc <- lapply(allX,function(x,y){return(log(prop.table(table(y,x)+1,1)))},y)
  #objet
  objet <- list()
  #liste des modalits
  objet$modalites <- levels(y)
  #nombre de modalits
  objet$nb_modalites <- nlevels(y)
  #log des probas a priori
  objet$prior <- as.numeric(log(prop.table(table(y)+1)))
  #nombre de variables explicatives
  objet$p <- ncol(allX)
  #noms de ces variables
  objet$var_names <- colnames(allX)
  #tableaux croiss
  objet$tc <- tc
  #classe
  class(objet) <- "NaiveBayesDiscrete"
  #return
  return(objet)
}

#prediction avec utilisation des indices
mypredict <- function(model,newdata){
  #vrifier les noms de variables
  if (length(intersect(model$var_names,colnames(newdata))) < model$p){stop("souci de variables")}
  #rcuprer les variables, et dans le bon ordre
  D <- newdata[model$var_names]
  #transformer les factors en code (indice de modalit)
  D <- data.frame(lapply(D,function(x){return(unclass(x))}))
  #matrice des scores - calcul par addition des log
  #triple boucles imbriques (indices) : (i) les lignes de newdata, (k) les modalits de la classe, (j) les colonnes (variables)
  #scores possde NB_Modalites lignes et NB_Examples(newdata) colonnes
  scores <- sapply(1:nrow(newdata),function(i,model,D){colSums(sapply(1:model$nb_modalites,function(k){sapply(1:model$p,function(j){model$tc[[j]][k,D[i,j]]})}))},model,D)
  #il faut rajouter les prior
  for (k in 1:model$nb_modalites){scores[k,] <- scores[k,]+model$prior[k]}
  #return(scores)
  #prediction
  mPred <- model$modalites[apply(scores,2,which.max)]
  return(mPred)
}

#modele
print(system.time(modele <- myfit(CLASSE ~ ., data = donnees)))
#print(modele$prior)
#print(modele$tc)

#mulation de boucles
print(system.time(pred1 <- mypredict(modele,newdata = donnees)))
#print(pred1)

#matrice de confusion
mc <- table(donnees$CLASSE,pred1)
print(mc)

#taux de reconnaissance
print(sum(diag(mc))/sum(mc))
