rm(list=ls())

donnees <- read.table("heart_for_naive_bayes.txt",header=T,sep="\t")

str(donnees)


#pour l'apprentissage
myfit <- function(formule,data){
  #vrification
  if (!inherits(formule,"formula")) {formule <- as.formula(formule)}
  #data.frame
  D <- model.frame(formule,data)
  #y
  y <- D[,1]
  #X
  allX <- D[,-1]
  #calcul des tableaux croiss
  tc <- lapply(allX,function(x,y){return(log(prop.table(table(y,x)+1,1)))},y)
  #objet
  objet <- list()
  #liste des modalits
  objet$modalites <- levels(y)
  #nombre de modalits
  objet$nb_modalites <- nlevels(y)
  #log des probas a priori
  objet$prior <- as.numeric(log(prop.table(table(y)+1)))
  #nombre de variables explicatives
  objet$p <- ncol(allX)
  #noms de ces variables
  objet$var_names <- colnames(allX)
  #tableaux croiss
  objet$tc <- tc
  #classe
  class(objet) <- "NaiveBayesDiscrete"
  #return
  return(objet)
}

#prediction - sans jamais utiliser des indices
fast_predict <- function(model,newdata){
  #vrifier les noms de variables
  if (length(intersect(model$var_names,colnames(newdata))) < model$p){stop("souci de variables")}
  #rcuprer les variables, et dans le bon ordre
  D <- newdata[model$var_names]
  #transformer les factors en code (indice de modalit)
  D <- as.matrix(data.frame(lapply(D,function(x){return(unclass(x))})))
  #matrice des scores (addition en log)
  #//!\\ instruction fatidique - utilisation de mapply  la place des boucles
  scores <- apply(D,1,function(ligne,tc){rowSums(mapply(function(value,tableau){return(tableau[,value])},ligne,tc))},model$tc)
  #rajouter les prior
  scores <- apply(scores,2,function(ligne,prior){return(mapply(sum,ligne,prior))},prior=model$prior)
  #return(scores)
  #prediction
  mPred <- model$modalites[apply(scores,2,which.max)]
  return(mPred)
}

#modele
print(system.time(modele <- myfit(disease ~ ., data = donnees)))
#print(modele$prior)
#print(modele$tc)

#prediction
print(system.time(pred2 <- fast_predict(modele,newdata = donnees)))
#print(pred2)

#matrice de confusion
mc <- table(donnees$disease,pred2)
print(mc)

#taux de reconnaissance
print(sum(diag(mc))/sum(mc))
