rm(list=ls())

donnees <- read.table("heart_for_naive_bayes.txt",header=T,sep="\t")
str(donnees)


#pour l'apprentissage
myfit <- function(formule,data){
  #vrification
  if (!inherits(formule,"formula")) {formule <- as.formula(formule)}
  #data.frame
  D <- model.frame(formule,data)
  #y
  y <- D[,1]
  #X
  allX <- D[,-1]
  #calcul des tableaux croiss
  tc <- lapply(allX,function(x,y){return(log(prop.table(table(y,x)+1,1)))},y)
  #nombre de modalits des X
  nbModaX <- sapply(allX,function(x){return(nlevels(x))})
  #//!\\conversion en matrice des log de proba
  K <- nlevels(y)
  MLog <- matrix(0,nrow=sum(nbModaX),ncol=K)
  numBloc <- 0
  #pour chaque tableau crois
  for (i in 1:length(tc)){
    temp <- tc[[i]]
    #pour chaque modalit de Y
    for (j in 1:K){
      MLog[(numBloc+1):(numBloc+nbModaX[i]),j] <- temp[j,]
    }
    numBloc <- numBloc + nbModaX[i]
  }
  #objet
  objet <- list()
  #liste des modalits
  objet$modalites <- levels(y)
  #nombre de modalits
  objet$nb_modalites <- K
  #log des probas a priori
  objet$prior <- as.numeric(log(prop.table(table(y)+1)))
  #nombre de variables explicatives
  objet$p <- ncol(allX)
  #noms de ces variables
  objet$var_names <- colnames(allX)
  #tableaux croiss
  objet$tc <- tc
  #nombre total de modalits des X
  objet$NbAllValues <- sum(nbModaX)
  #tableau de dcalage des modalits
  objet$DecValues <- cumsum(c(0,nbModaX[-length(nbModaX)]))
  #matrice pour la projection
  objet$MLog <- MLog
  #classe
  class(objet) <- "NaiveBayesDiscrete"
  #return
  return(objet)
}

#prediction - sans jamais utiliser des indices
quick_predict <- function(model,newdata){
  #vrifier les noms de variables
  if (length(intersect(model$var_names,colnames(newdata))) < model$p){stop("souci de variables")}
  #rcuprer les variables, et dans le bon ordre
  D <- newdata[model$var_names]
  #transformer les factors en code (indice de modalit)
  D <- as.matrix(data.frame(lapply(D,function(x){return(unclass(x))})))
  #//!\\ transformer D en matrice de codes binaires -- attention inflation de colonnes si nombreuses modalits
  M <- t(apply(D,1,function(x){v <- rep(0,model$NbAllValues);v[model$DecValues+x] <- 1; return(v)}))
  #//!\\ simple produit matriciel pour la prdiction
  scores <- M %*% model$MLog
  #il faut rajouter les prior
  for (k in 1:model$nb_modalites){scores[,k] <- scores[,k]+model$prior[k]}
  #return(scores)
  #prediction
  mPred <- model$modalites[apply(scores,1,which.max)]
  return(mPred)
}

#modele
print(system.time(modele <- myfit(disease ~ ., data = donnees)))
print(modele$prior)
#print(modele$tc)
#print(modele$MLog)

#prediction
print(system.time(pred <- quick_predict(modele,newdata = donnees)))
#print(pred)

#matrice de confusion
mc <- table(donnees$disease,pred)
print(mc)

#taux de reconnaissance
print(sum(diag(mc))/sum(mc))
